@@ -1,7 +1,7 @@
 
 /* sg.c
    Switcher for operating system dependent transport level modules of libburn.
-   Copyright (C) 2009 - 2010 Thomas Schmitt <scdbackup@gmx.net>, 
+   Copyright (C) 2009 - 2014 Thomas Schmitt <scdbackup@gmx.net>, 
    provided under GPLv2+
 */
 
@@ -11,6 +11,12 @@
 #endif
 
 
+/* <<< Until it is known whether this adapter would work on OpenBSD too */
+#ifdef __NetBSD__
+#define Libburn_use_sg_netbsD
+#endif
+
+
 #ifdef Libburn_use_sg_dummY
 
 #include "sg-dummy.c"
@@ -20,15 +26,29 @@
 
 #include "sg-libcdio.c"
 
+#else
+#ifdef Libburn_use_sg_netbsD
+/* To become: # ifdef __NetBSD__ */
+
+#include "sg-netbsd.c"
+
 #else
 #ifdef __FreeBSD__
 
+#ifdef Libburn_use_sg_freebsd_porT
+#include "sg-freebsd-port.c"
+#else
 #include "sg-freebsd.c"
+#endif
 
 #else
 #ifdef __FreeBSD_kernel__
 
+#ifdef Libburn_use_sg_freebsd_porT
+#include "sg-freebsd-port.c"
+#else
 #include "sg-freebsd.c"
+#endif
 
 #else
 #ifdef __linux
@@ -50,11 +70,11 @@
 static int intentional_compiler_warning(void)
 {
  int INTENTIONAL_COMPILER_WARNING_;
- int Cannot_recognize_GNU_Linux_nor_FreeBSD_;
+ int Cannot_recognize_GNU_Linux_nor_FreeBSD_nor_Solaris_nor_NetBSD_;
  int Have_to_use_dummy_MMC_transport_adapter_;
  int This_libburn_will_not_be_able_to_operate_on_real_CD_drives;
  int Have_to_use_dummy_MMC_transport_adapter;
- int Cannot_recognize_GNU_Linux_nor_FreeBSD;
+ int Cannot_recognize_GNU_Linux_nor_FreeBSD_nor_Solaris_nor_NetBSD;
  int INTENTIONAL_COMPILER_WARNING;
 
  return(0);
@@ -66,6 +86,7 @@ static int intentional_compiler_warning(void)
 #endif /* ! __linux */
 #endif /* ! __FreeBSD_kernel__ */
 #endif /* ! __FreeBSD__ */
+#endif /* ! Libburn_use_sg_netbsD */
 #endif /* ! Libburn_use_libcdiO */
 #endif /* ! Libburn_use_sg_dummY */
 
