@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -282,11 +282,11 @@ int Sectorbitmap_destroy(struct SectorbitmaP **o, int flag)
 int Sectorbitmap_from_file(struct SectorbitmaP **o, char *path, char *msg,
                            int *os_errno, int flag)
 {
- int ret, fd= -1, sectors, sector_size, i, todo, map_size, skip;
+ int ret, fd= -1, sectors, sector_size, i, todo, map_size, skip, bufsize= 1024;
  unsigned char *map;
  unsigned char *buf;
 
- buf= TSOB_FELD(unsigned char, 1024);
+ buf= TSOB_FELD(unsigned char, bufsize);
  if(buf == NULL)
    return(-1);
 
@@ -320,8 +320,8 @@ wrong_filetype:;
    sscanf(((char *) buf) + 25, "%d", &skip);
    if(skip < 0)
      {ret= 0; goto wrong_filetype;}
-   for(i= 0; i < skip; i+= sizeof(buf)) {
-     todo= sizeof(buf);
+   for(i= 0; i < skip; i+= bufsize) {
+     todo= bufsize;
      if(i + todo > skip)
        todo= skip - i;
      ret= read(fd, buf, todo);
@@ -345,8 +345,8 @@ wrong_filetype:;
  }
  map= (*o)->map;
  map_size= (*o)->map_size;
- for(i= 0; i < map_size; i+= sizeof(buf)) {
-   todo=  sizeof(buf);
+ for(i= 0; i < map_size; i+= bufsize) {
+   todo= bufsize;
    if(i + todo > map_size)
      todo= map_size - i;
    ret= read(fd, buf, todo);
@@ -571,6 +571,7 @@ int Checkmediajob_new(struct CheckmediajoB **o, int flag)
  m->min_lba= -1;
  m->max_lba= -1;
  m->min_block_size= 0;
+ m->async_chunks= 0;
  m->mode= 0;
  m->start_time= time(NULL);
  m->time_limit= 28800;
@@ -589,7 +590,6 @@ int Checkmediajob_new(struct CheckmediajoB **o, int flag)
  m->report_mode= 0;
  strcpy(m->event_severity, "ALL");
  m->slow_threshold_seq= 1.0;
- m->untested_valid= 0;
  return(1);
 }
 
@@ -631,7 +631,6 @@ int Checkmediajob_copy(struct CheckmediajoB *from, struct CheckmediajoB *to,
  to->report_mode= from->report_mode;
  strcpy(to->event_severity, from->event_severity);
  to->slow_threshold_seq= from->slow_threshold_seq;
- to->untested_valid= from->untested_valid;
  return(1);
 }
 
@@ -655,6 +654,12 @@ int Xorriso_check_media_setup_job(struct XorrisO *xorriso,
      ret= Sfile_str(job->abort_file_path, argv[i] + 11, 0);
      if(ret <= 0)
        goto ex;
+   } else if(strncmp(argv[i], "async_chunks=", 13) == 0) {
+     num= Scanf_io_size(argv[i] + 13, 1);
+     if(num >= 0 && num <= 1024)
+       job->async_chunks= num;
+     else
+       goto bad_value;
    } else if(strncmp(argv[i], "bad_limit=", 10) == 0) {
      if(strcmp(argv[i] + 10, "good") == 0)
        xorriso->check_media_bad_limit= Xorriso_read_quality_gooD;
@@ -689,7 +694,7 @@ int Xorriso_check_media_setup_job(struct XorrisO *xorriso,
      if(num >= 2048 || num == 0)
        job->min_block_size= num / 2048;
      else
-       goto unknown_value;
+       goto bad_value;
    } else if(strncmp(argv[i], "event=", 6) == 0) {
      strncpy(sev_text, argv[i] + 6, 19);
      sev_text[19]= 0;
@@ -731,7 +736,7 @@ int Xorriso_check_media_setup_job(struct XorrisO *xorriso,
        num= -1;
        sscanf(argv[i] + 11, "%lf", &num);
        if(num > 0x7fffffff || num < 0)
-         goto unknown_value;
+         goto bad_value;
        job->patch_lba0_msc1= num;
        job->patch_lba0= (num >= 32) + (strstr(argv[i] + 11, ":force") != NULL);
      } else 
@@ -781,18 +786,6 @@ int Xorriso_check_media_setup_job(struct XorrisO *xorriso,
        job->time_limit= num;
      else
        job->item_limit= num;
-
-#ifdef NIX
-   } else if(strncmp(argv[i], "untested=", 9) == 0) {
-     if(strcmp(argv[i] + 9, "damaged") == 0)
-       job->untested_valid= 0;
-     if(strcmp(argv[i] + 9, "undamaged") == 0 ||
-        strcmp(argv[i] + 9, "ok") == 0)
-       job->untested_valid= 1;
-     else
-       goto unknown_value;
-#endif
-
    } else if(strncmp(argv[i], "use=", 4) == 0) {
      if(strcmp(argv[i] + 4, "outdev") == 0)
        job->use_dev= 1;
@@ -805,14 +798,21 @@ int Xorriso_check_media_setup_job(struct XorrisO *xorriso,
    } else if(strncmp(argv[i], "what=", 5) == 0) {
      if(strcmp(argv[i]+5, "tracks") == 0)
        job->mode= 0;
-     else if(strcmp(argv[i]+5, "disc")== 0)
+     else if(strcmp(argv[i]+5, "image")== 0)
        job->mode= 1;
+     else if(strcmp(argv[i]+5, "disc")== 0)
+       job->mode= 2;
      else {
 unknown_value:;
        sprintf(xorriso->info_text,
                "-check_media: Unknown value with option %s", argv[i]);
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
        ret= 0; goto ex;
+bad_value:;
+       sprintf(xorriso->info_text,
+               "-check_media: Unsuitable value with option %s", argv[i]);
+       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+       ret= 0; goto ex;
      }
    } else {
      sprintf(xorriso->info_text, "-check_media: Unknown option '%s'", argv[i]);
@@ -991,6 +991,7 @@ ex:;
 
 
 /* @param flag bit0= mark untested areas as valid
+               bit1= leave untested areas as they are
 */
 int Xorriso_spotlist_to_sectormap(struct XorrisO *xorriso,
                                   struct SpotlisT *spotlist,
@@ -1050,7 +1051,9 @@ int Xorriso_spotlist_to_sectormap(struct XorrisO *xorriso,
      valid= quality > xorriso->check_media_bad_limit;
      if(quality == Xorriso_read_quality_untesteD && (flag & 1))
        valid= 1;
-     else if(pass == 0 && !valid)
+     if(quality == Xorriso_read_quality_untesteD && (flag & 2))
+   continue;
+     if(pass == 0 && !valid)
    continue;
      else if(pass == 1 && valid)
    continue;
