@@ -202,10 +202,10 @@ cannot_address:;
      xorriso->pacifier_byte_count+= r1;
      if(flag&(1<<31))
        Xorriso_pacifier_callback(xorriso, "content bytes read",
-                                 xorriso->pacifier_count, 0, "", 0);
+                                 xorriso->pacifier_count, 0, "", 8);
      else
        Xorriso_pacifier_callback(xorriso, "bytes", xorriso->pacifier_count, 0,
-                                 "", 1<<6);
+                                 "", 8 | 1<<6);
    }
  }
 
