@@ -382,6 +382,7 @@ int Findjob_new(struct FindjoB **o, char *start_path, int flag)
  m->subjob= NULL;
  m->errmsg[0]= 0;
  m->errn= 0;
+ m->match_count= 0;
 
  ret= Exprnode_new(&(m->test_tree), m, NULL, "-find", (m->use_shortcuts)<<1);
  if(ret<=0)
@@ -758,7 +759,7 @@ improper_range:;
 }
 
 
-/* @param flag bit0-1=  0= -name , 1= -wholename , 2= -disk_name
+/* @param flag bit0-1: 0= -name , 1= -wholename , 2= -disk_name , 3= -disk_path
 */
 int Findjob_set_name_expr(struct FindjoB *o, char *name_expr, int flag)
 {
@@ -782,14 +783,18 @@ int Findjob_set_name_expr(struct FindjoB *o, char *name_expr, int flag)
    t->test_type= 13;
  else if((flag & 3) == 2)
    t->test_type= 16;
+ else if((flag & 3) == 3)
+   t->test_type= 20;
+ t->arg1= strdup(name_expr);
+ if(t->arg1 == NULL)
+   {ret= -1; goto ex;};
+
+ if((flag & 3) == 3)
+   {ret= 1; goto ex;}
+
  name_re= (regex_t *) calloc(1, sizeof(regex_t));
  if(name_re == NULL)
    {ret= -1; goto ex;};
- t->arg1= strdup(name_expr);
- if(t->arg1 == NULL) {
-   free((char *) name_re);
-   {ret= -1; goto ex;};
- }
  Xorriso__bourne_to_reg(name_expr, regexpr, 0);
  if(regcomp(name_re, regexpr, 0) != 0) {
    free((char *) name_re);
@@ -859,8 +864,8 @@ int Findjob_set_damage_filter(struct FindjoB *o, int value, int flag)
 }
  
 
-int Findjob_set_lba_range(struct FindjoB *o, int start_lba, int count,
-                          int flag)
+int Findjob_set_num_filter(struct FindjoB *o, int test_type,
+                           int num1, int num2, int flag)
 {
  struct ExprtesT *t;
  int ret;
@@ -870,20 +875,31 @@ int Findjob_set_lba_range(struct FindjoB *o, int start_lba, int count,
    return(ret);
 
  t= o->cursor->test;
- t->test_type= 4;
+ t->test_type= test_type;
  t->arg1= calloc(sizeof(int), 1);
  t->arg2= calloc(sizeof(int), 1);
  if(t->arg1 == NULL || t->arg2 == NULL)
    return(-1);
- *((int *) t->arg1)= start_lba;
- if(start_lba > 0)
-   *((int *) t->arg2)= start_lba + count - 1;
- else
-   *((int *) t->arg2)= start_lba - count + 1;
+ *((int *) t->arg1)= num1;
+ *((int *) t->arg2)= num2;
  return(1);
 }
 
 
+int Findjob_set_lba_range(struct FindjoB *o, int start_lba, int count,
+                          int flag)
+{
+ int ret, end_lba;
+
+ if(start_lba > 0)
+   end_lba= start_lba + count - 1;
+ else
+   end_lba= start_lba - count + 1;
+ ret= Findjob_set_num_filter(o, 4, start_lba, end_lba, 0);
+ return(ret);
+}
+
+
 int Findjob_set_test_hidden(struct FindjoB *o, int mode, int flag)
 {
  struct ExprtesT *t;
@@ -948,6 +964,51 @@ int Findjob_set_filter_filter(struct FindjoB *o, int value, int flag)
 }
 
 
+int Findjob_set_crtp_filter(struct FindjoB *o, char *creator, char *hfs_type,
+                            int flag)
+{
+ struct ExprtesT *t;
+ int ret;
+
+ ret= Findjob_default_and(o, 0);
+ if(ret <= 0)
+   return(ret);
+
+ t= o->cursor->test;
+ t->test_type= 18;
+ t->arg1= calloc(1, strlen(creator) + 1);
+ t->arg2= calloc(1, strlen(hfs_type) + 1);
+ if(t->arg1 == NULL || t->arg2 == NULL)
+   return(-1);
+ strcpy(t->arg1, creator);
+ strcpy(t->arg2, hfs_type);
+ return(1);
+}
+
+
+int Findjob_set_bless_filter(struct XorrisO *xorriso, struct FindjoB *o,
+                             char *blessing, int flag)
+{
+ struct ExprtesT *t;
+ int ret;
+
+ ret= Findjob_default_and(o, 0);
+ if(ret <= 0)
+   return(ret);
+
+ t= o->cursor->test;
+ t->test_type= 19;
+ t->arg1= calloc(1, sizeof(int));
+ if(t->arg1 == NULL)
+   return(-1);
+ ret= Xorriso_hfsplus_bless(xorriso, "", NULL, blessing, 4 | 8);
+ if(ret <= 0)
+   return(ret);
+ *((int *) t->arg1)= ret - 1;
+ return(1);
+}
+
+
 int Findjob_set_wanted_node(struct FindjoB *o, void *wanted_node, int flag)
 {
  struct ExprtesT *t;
