@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -33,6 +33,10 @@
 /* Option -pacifier */
 int Xorriso_option_pacifier(struct XorrisO *xorriso, char *style, int flag)
 {
+
+#define Xorriso_pacifier_min_intvL 0.1
+#define Xorriso_pacifier_max_intvL 60.0
+
  if(strcmp(style, "xorriso")==0 || strcmp(style, "default")==0)
    xorriso->pacifier_style= 0;
  else if(strcmp(style, "mkisofs")==0 || strcmp(style, "genisofs")==0 ||
@@ -41,7 +45,24 @@ int Xorriso_option_pacifier(struct XorrisO *xorriso, char *style, int flag)
  else if(strcmp(style, "cdrecord")==0 || strcmp(style, "cdrskin")==0 ||
          strcmp(style, "wodim")==0 || strcmp(style, "xorrecord")==0)
    xorriso->pacifier_style= 2;
- else {
+
+ else if(strncmp(style, "interval=", 9) == 0) {
+   sscanf(style + 9, "%lf", &(xorriso->pacifier_interval));
+   if(xorriso->pacifier_interval < Xorriso_pacifier_min_intvL) {
+     sprintf(xorriso->info_text,
+             "-pacifier: interval='%s' is too low. Min: %f",
+             style, Xorriso_pacifier_min_intvL);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+     xorriso->pacifier_interval= Xorriso_pacifier_min_intvL;
+   } else if(xorriso->pacifier_interval > Xorriso_pacifier_max_intvL) {
+     sprintf(xorriso->info_text,
+             "-pacifier: interval='%s' is too high. Max: %f",
+             style, Xorriso_pacifier_max_intvL);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+     xorriso->pacifier_interval= Xorriso_pacifier_max_intvL;
+   }
+
+ } else {
    sprintf(xorriso->info_text, "-pacifier: unknown behavior code '%s'", style);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
    return(0);
@@ -293,7 +314,7 @@ int Xorriso_option_print_size(struct XorrisO *xorriso, int flag)
 {
  int ret, fd;
 
- if(!xorriso->volset_change_pending) {
+ if(!Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text,"-print_size: No image modifications pending");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
    if(!(flag & 1)) {
@@ -395,8 +416,10 @@ int Xorriso_option_pwdi(struct XorrisO *xorriso, int flag)
 {
  sprintf(xorriso->info_text,"current working directory in ISO image:\n");
  Xorriso_info(xorriso,0);
- Text_shellsafe(xorriso->wdi, xorriso->result_line, 0);
- strcat(xorriso->result_line, "/\n");
+ Xorriso_esc_filepath(xorriso, xorriso->wdi, xorriso->result_line, 0);
+ if(xorriso->sh_style_result == 0 || xorriso->wdi[0] == 0)
+   strcat(xorriso->result_line, "/");
+ strcat(xorriso->result_line, "\n");
  Xorriso_result(xorriso,0);
  return(1);
 }
@@ -407,7 +430,10 @@ int Xorriso_option_pwdx(struct XorrisO *xorriso, int flag)
 {
  sprintf(xorriso->info_text,"current working directory on hard disk:\n");
  Xorriso_info(xorriso,0);
- sprintf(xorriso->result_line,"%s/\n",xorriso->wdx);
+ Xorriso_esc_filepath(xorriso, xorriso->wdx, xorriso->result_line, 0);
+ if(xorriso->sh_style_result == 0 || xorriso->wdx[0] == 0)
+   strcat(xorriso->result_line, "/");
+ strcat(xorriso->result_line, "\n");
  Xorriso_result(xorriso,0);
  return(1);
 }
@@ -440,31 +466,25 @@ int Xorriso_option_reassure(struct XorrisO *xorriso, char *mode, int flag)
 }
 
 
-/* Option -unregister_filter */
-int Xorriso_option_unregister_filter(struct XorrisO *xorriso, char *name,
-                                     int flag)
-{
- int ret;
-
- ret= Xorriso_external_filter(xorriso, name, "", "", 0, NULL, 1);
- return(ret);
-}
-
-
 /* Option -report_about */
-int Xorriso_option_report_about(struct XorrisO *xorriso, char *severity,
+int Xorriso_option_report_about(struct XorrisO *xorriso, char *in_severity,
                                 int flag)
 {
  int ret, sev;
+ char severity[20], *official;
 
+ Xorriso__to_upper(in_severity, severity, (int) sizeof(severity), 0);
  ret= Xorriso__text_to_sev(severity, &sev, 0);
  if(ret<=0) {
    sprintf(xorriso->info_text, "-report_about: Not a known severity name : ");
-   Text_shellsafe(severity, xorriso->info_text, 1);
+   Text_shellsafe(in_severity, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
    return(ret);
  }
- if(Sfile_str(xorriso->report_about_text,severity,0)<=0)
+ ret= Xorriso__sev_to_text(sev, &official, 0);
+ if(ret <= 0)
+   official= severity;
+ if(Sfile_str(xorriso->report_about_text, official, 0) <= 0)
    return(-1);
  xorriso->report_about_severity= sev;
  return(1);
@@ -472,26 +492,31 @@ int Xorriso_option_report_about(struct XorrisO *xorriso, char *severity,
 
 
 /* Option -return_with */
-int Xorriso_option_return_with(struct XorrisO *xorriso, char *severity,
+int Xorriso_option_return_with(struct XorrisO *xorriso, char *in_severity,
                                int exit_value, int flag)
 {
  int ret, sev;
+ char severity[20], *official;
 
+ Xorriso__to_upper(in_severity, severity, (int) sizeof(severity), 0);
  ret= Xorriso__text_to_sev(severity, &sev, 0);
  if(ret<=0) {
    sprintf(xorriso->info_text,
            "-return_with: Not a known severity name : ");
-   Text_shellsafe(severity, xorriso->info_text, 1);
+   Text_shellsafe(in_severity, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(ret);
  }
+ ret= Xorriso__sev_to_text(sev, &official, 0);
+ if(ret <= 0)
+   official= severity;
  if(exit_value && (exit_value < 32 || exit_value > 63)) {
    sprintf(xorriso->info_text,
            "-return_with: Not an allowed exit_value. Use 0, or 32 to 63.");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(0);
  }
- if(Sfile_str(xorriso->return_with_text,severity,0)<=0)
+ if(Sfile_str(xorriso->return_with_text, official, 0) <= 0)
    return(-1);
  xorriso->return_with_severity= sev;
  xorriso->return_with_value= exit_value;
@@ -566,6 +591,22 @@ ex:;
 }
 
 
+/* Option -rockridge "on"|"off" */
+int Xorriso_option_rockridge(struct XorrisO *xorriso, char *mode, int flag)
+{
+ if(strcmp(mode, "off")==0)
+   xorriso->do_rockridge= 0;
+ else if(strcmp(mode, "on")==0)
+   xorriso->do_rockridge= 1;
+ else {
+   sprintf(xorriso->info_text, "-rockridge: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -rollback */
 /* @param flag bit0= do not -reassure
    @return <=0 error , 1 success, 2 revoked by -reassure
@@ -576,7 +617,7 @@ int Xorriso_option_rollback(struct XorrisO *xorriso, int flag)
  char *indev= NULL, *which_will;
 
  Xorriso_alloc_meM(indev, char, SfileadrL);
- if(xorriso->volset_change_pending)
+ if(Xorriso_change_is_pending(xorriso, 0))
    which_will= "revoke the pending image changes";
  else
    which_will= "reload the image";
@@ -588,6 +629,7 @@ int Xorriso_option_rollback(struct XorrisO *xorriso, int flag)
 
  if(Sfile_str(indev, xorriso->indev, 0)<=0)
    {ret= -1; goto ex;}
+ xorriso->volset_change_pending= 0;
  ret= Xorriso_give_up_drive(xorriso, 1|8);
  if(ret<=0)
    goto ex;
@@ -605,7 +647,6 @@ int Xorriso_option_rom_toc_scan(struct XorrisO *xorriso, char *mode, int flag)
  int l;
  char *cpt, *npt;
 
- xorriso->toc_emulation_flag= 0;
  npt= cpt= mode;
  for(cpt= mode; npt != NULL; cpt= npt + 1) {
    npt= strchr(cpt,':');
@@ -614,20 +655,22 @@ int Xorriso_option_rom_toc_scan(struct XorrisO *xorriso, char *mode, int flag)
    else
      l= npt-cpt;
    if(l==0)
-     goto unknown_mode;
-   xorriso->toc_emulation_flag&= ~4;
+ continue;
    if(strncmp(cpt, "off", l) == 0)
-     xorriso->toc_emulation_flag&= ~1;
+     xorriso->toc_emulation_flag&= ~5;
    else if(strncmp(cpt, "on", l) == 0)
-     xorriso->toc_emulation_flag|= 1;
+     xorriso->toc_emulation_flag= (xorriso->toc_emulation_flag & ~4) | 1;
    else if(strncmp(cpt, "force", l) == 0)
-     xorriso->toc_emulation_flag|= 4;
+     xorriso->toc_emulation_flag|= 5;
    else if(strncmp(cpt, "emul_off", l) == 0)
      xorriso->toc_emulation_flag|= 2;
    else if(strncmp(cpt, "emul_on", l) == 0)
      xorriso->toc_emulation_flag&= ~2;
+   else if(strncmp(cpt, "emul_wide", l) == 0)
+     xorriso->toc_emulation_flag|= 8;
+   else if(strncmp(cpt, "emul_narrow", l) == 0)
+     xorriso->toc_emulation_flag&= ~8;
    else {
-unknown_mode:;
      sprintf(xorriso->info_text, "-rom_toc_scan: unknown mode in '%s'", mode);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
      return(0);
@@ -637,6 +680,26 @@ unknown_mode:;
 }
 
 
+/* Command -rr_reloc_dir */
+int Xorriso_option_rr_reloc_dir(struct XorrisO *xorriso, char *name, int flag)
+{
+ if(strlen(name) > 255) {
+   sprintf(xorriso->info_text,
+           "Name too long with -rr_reloc_dir. Max. 255 bytes allowed.");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ if(strchr(name, '/') != NULL) {
+   sprintf(xorriso->info_text,
+           "Name given with -rr_reloc_dir contains '/' character");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ strcpy(xorriso->rr_reloc_dir, name);
+ return(1);
+}
+
+
 /* Option -scdbackup_tag list_path record_name */
 int Xorriso_option_scdbackup_tag(struct XorrisO *xorriso, char *listname,
                                  char *recname, int flag)
@@ -681,7 +744,6 @@ int Xorriso_option_session_log(struct XorrisO *xorriso, char *path, int flag)
 }
 
 
-
 /* Option -setfacl_list alias -setfacl_listi */
 int Xorriso_option_setfacl_listi(struct XorrisO *xorriso, char *path, int flag)
 {
@@ -1166,6 +1228,23 @@ ex:;
 }
 
 
+/* Option -sh_style_result */
+int Xorriso_option_sh_style_result(struct XorrisO *xorriso, char *mode,
+                                   int flag)
+{
+ if(strcmp(mode, "off") == 0) {
+   xorriso->sh_style_result= 0;
+ } else if(strcmp(mode, "on") == 0) {
+   xorriso->sh_style_result= 1;
+ } else {
+   sprintf(xorriso->info_text, "-sh_style_result: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1); 
+}
+
+
 /* Option -signal_handling */
 /* @param flag bit0= prescan mode: do not yet install the eventual handler
                      else: when switching from other modes to "off":
@@ -1207,19 +1286,66 @@ int Xorriso_option_signal_handling(struct XorrisO *xorriso, char *mode,
 }
 
 
-/* Option -speed */
+/* Option -sleep */
+int Xorriso_option_sleep(struct XorrisO *xorriso, char *duration, int flag)
+{
+ double dur= 0.0, start_time, end_time, todo, granularity= 0.01;
+ unsigned long usleep_time; 
+
+ sscanf(duration, "%lf", &dur);
+ start_time= Sfile_microtime(0);
+ end_time= start_time + dur;
+ Ftimetxt(time(NULL), xorriso->info_text, 6);
+ sprintf(xorriso->info_text + strlen(xorriso->info_text),
+         " : Will sleep for %f seconds", dur);
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "UPDATE", 0);
+ while(1) {
+   todo= end_time - Sfile_microtime(0);
+   if(todo <= 0)
+     usleep_time= 0;
+   else if(todo > granularity)
+     usleep_time= granularity * 1.0e6;
+   else
+     usleep_time= todo * 1.0e6;
+   if(usleep_time == 0)
+ break;
+   usleep(usleep_time);
+ }
+ sprintf(xorriso->info_text, "Slept for %f seconds",
+         Sfile_microtime(0) - start_time);
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+ return(1);
+}
+
+
+/* Commands -speed , -read_speed */
+/* @param flag bit0= -read_speed rather than -speed
+*/
 int Xorriso_option_speed(struct XorrisO *xorriso, char *speed, int flag)
 {
- int is_cd= 1, unit_found= 0, ret, profile_number;
- double num;
+ int is_cd= 1, unit_found= 0, ret, profile_number, intspeed= 1;
+ double num= -2.0;
  char *cpt, profile_name[80];
 
- if(speed[0]==0 || strcmp(speed, "any")==0) {
-   xorriso->speed= 0; /* full speed */
+ if(speed[0] == 0 || strcmp(speed, "any") == 0 || strcmp(speed, "max") == 0) {
+   intspeed= 0;
+ } else if(strcmp(speed, "min") == 0) {
+   intspeed= -1;
+ } else if(strcmp(speed, "none") == 0) {
+   intspeed= -2;
+ } else {
+   sscanf(speed,"%lf",&num);
+   if(num <= 0)
+     intspeed= num;
+ }
+ if(intspeed <= 0) {
+   if(flag & 1)
+     xorriso->read_speed= intspeed;
+   else
+     xorriso->write_speed= intspeed;
    return(1);
  }
 
- sscanf(speed,"%lf",&num);
  for(cpt= speed+strlen(speed)-1; cpt>=speed; cpt--)
    if(isdigit(*cpt) || *cpt=='.')
  break;
@@ -1246,7 +1372,8 @@ dvd_speed:;
 bd_speed:;
    num*= 4495.625;
  } else {
-   ret= Xorriso_get_profile(xorriso, &profile_number, profile_name, 2);
+   ret= Xorriso_get_profile(xorriso, &profile_number, profile_name,
+                            2 * !(flag & 1));
    is_cd= (ret==2);
    if(is_cd)
      goto cd_speed;
@@ -1262,9 +1389,13 @@ bd_speed:;
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
    return(0);
  }
- xorriso->speed= num;
- if(xorriso->speed<num)
-   xorriso->speed++;
+ intspeed= num;
+ if(intspeed < num)
+   intspeed++;
+ if(flag & 1)
+   xorriso->read_speed= intspeed;
+ else
+   xorriso->write_speed= intspeed;
  return(1);
 }
 
@@ -1388,7 +1519,7 @@ int Xorriso_option_tell_media_space(struct XorrisO *xorriso, int flag)
  }
  if(free_space<0) {
    sprintf(xorriso->info_text,
-           "Pending image size larger than free space on media");
+           "Pending image size larger than free space on medium");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
  }
  sprintf(xorriso->result_line, "Media space  : %ds\n", media_space);
@@ -1419,6 +1550,8 @@ int Xorriso_option_temp_mem_limit(struct XorrisO *xorriso, char *size,
 
 
 /* Option -toc */
+/* @param flag   bit0= short report form as with -dev, no table-of-content
+*/
 int Xorriso_option_toc(struct XorrisO *xorriso, int flag)
 {
  int ret, in_ret= 1000;
@@ -1434,7 +1567,7 @@ int Xorriso_option_toc(struct XorrisO *xorriso, int flag)
    }
    ret= 1;
    if(xorriso->outdev[0]!=0)
-     ret= Xorriso_toc(xorriso, 2);
+     ret= Xorriso_toc(xorriso, 2 | (flag & 1));
    if(in_ret<ret)
      ret= in_ret;
  }
@@ -1442,6 +1575,44 @@ int Xorriso_option_toc(struct XorrisO *xorriso, int flag)
 }
 
 
+/* Option -toc_of */
+int Xorriso_option_toc_of(struct XorrisO *xorriso, char *which, int flag)
+{
+ int ret= 0, toc_flag= 0;
+
+ if(strstr(which, ":short") != NULL)
+   toc_flag|= 1;
+ if(strncmp(which, "in", 2) == 0) {
+   if(xorriso->indev[0] == 0) {
+     Xorriso_msgs_submit(xorriso, 0, "-toc_of 'in' : No input drive aquired",
+                         0, "NOTE", 0);
+     return(2);
+   }
+   ret= Xorriso_toc(xorriso, toc_flag | 0);
+ } else if(strncmp(which, "out", 3) == 0) {
+   if(xorriso->outdev[0] == 0) {
+     Xorriso_msgs_submit(xorriso, 0, "-toc_of 'out' : No output drive aquired",
+                         0, "NOTE", 0);
+     return(2);
+   }
+   ret= Xorriso_toc(xorriso, toc_flag | 2);
+ } else if(strncmp(which, "all", 3) == 0) {
+   if(xorriso->indev[0] == 0 && xorriso->outdev[0] == 0) {
+     Xorriso_msgs_submit(xorriso, 0, "-toc_of 'all' : No drive aquired",
+                         0, "NOTE", 0);
+     return(2);
+   }
+   ret= Xorriso_option_toc(xorriso, toc_flag | 0);
+ } else {
+   sprintf(xorriso->info_text, "-toc_of: Unknown drive code ");
+   Text_shellsafe(which, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   ret= 0;
+ }
+ return(ret);
+}
+
+
 /* Option -uid */
 int Xorriso_option_uid(struct XorrisO *xorriso, char *uid, int flag)
 {
@@ -1457,6 +1628,17 @@ int Xorriso_option_uid(struct XorrisO *xorriso, char *uid, int flag)
 }
 
 
+/* Option -unregister_filter */
+int Xorriso_option_unregister_filter(struct XorrisO *xorriso, char *name,
+                                     int flag)
+{
+ int ret;
+
+ ret= Xorriso_external_filter(xorriso, name, "", "", 0, NULL, 1);
+ return(ret);
+}
+
+
 /* Options -update and -update_r
    @param flag bit0= issue start and summary message
                bit1= do not reset pacifier, no final pacifier message
@@ -1470,8 +1652,8 @@ int Xorriso_option_uid(struct XorrisO *xorriso, char *uid, int flag)
 int Xorriso_option_update(struct XorrisO *xorriso, char *disk_path,
                           char *iso_path, int flag)
 {
- int ret, mem_pci, zero= 0, result, uret, follow_links, do_register= 1;
- int not_in_iso= 0, not_on_disk= 0, made_di_array= 0;
+ int ret, mem_pci, zero= 0, result, uret, follow_links, do_register= 0;
+ int not_in_iso= 0, not_on_disk= 0;
  double mem_lut= 0.0, start_time;
  char *ipth, *argv[6];
  char *eff_origin= NULL, *eff_dest= NULL;
@@ -1524,12 +1706,12 @@ int Xorriso_option_update(struct XorrisO *xorriso, char *disk_path,
  if(!(xorriso->ino_behavior & 2)) {
    if(!(xorriso->di_array != NULL || (flag & 16))) {
      /* Create all-image node array sorted by isofs.di */
-     made_di_array= 1;
      ret= Xorriso_make_di_array(xorriso, 0);
      if(ret <= 0)
        goto ex;
    }
    if(xorriso->di_array != NULL) {
+     do_register= 1;
      if(!(flag & 8)) {
        /* If directory with -update : do not register di_*_paths */
        ret= lstat(eff_origin, &stbuf);
@@ -1537,16 +1719,6 @@ int Xorriso_option_update(struct XorrisO *xorriso, char *disk_path,
          if(S_ISDIR(stbuf.st_mode))
            do_register= 0;
      }
-     if(do_register) {
-       ret= Xorriso_lst_new(&(xorriso->di_disk_paths), eff_origin,
-                            xorriso->di_disk_paths, 1);
-       if(ret <= 0)
-         goto ex;
-       ret= Xorriso_lst_new(&(xorriso->di_iso_paths), eff_dest,
-                            xorriso->di_iso_paths, 1);
-       if(ret <= 0)
-         goto ex;
-     }
    }
  }
 
@@ -1638,16 +1810,34 @@ int Xorriso_option_update(struct XorrisO *xorriso, char *disk_path,
  xorriso->pacifier_interval= mem_pci;
  if(mem_lut!=xorriso->last_update_time && !(flag & (2 | 4)))
    Xorriso_pacifier_callback(xorriso, "content bytes read",
-                             xorriso->pacifier_count, 0, "", 1);
+                             xorriso->pacifier_count, 0, "", 1 | 8 | 32);
 report_outcome:;
  if(ret>0) {
    sprintf(xorriso->info_text,
            "No file object needed update.");
+   do_register= 0;
  } else if(ret==0) {
    sprintf(xorriso->info_text, "Differences detected and updated.");
  } else {
    sprintf(xorriso->info_text,
            "Not ok. Comparison or update failed due to error.");
+   do_register= 0;
+ }
+
+ if(do_register) {
+   ret= Xorriso_iso_lstat(xorriso, eff_dest, &stbuf, 0);
+   if(ret < 0)
+     do_register= 0;
+ }
+ if(do_register) {
+   ret= Xorriso_lst_new(&(xorriso->di_disk_paths), eff_origin,
+                        xorriso->di_disk_paths, 1);
+   if(ret <= 0)
+     goto ex;
+   ret= Xorriso_lst_new(&(xorriso->di_iso_paths), eff_dest,
+                        xorriso->di_iso_paths, 1);
+   if(ret <= 0)
+     goto ex;
  }
  sprintf(xorriso->info_text+strlen(xorriso->info_text),
          " (runtime %.1f s)\n", Sfile_microtime(0)-start_time);
@@ -1689,15 +1879,16 @@ int Xorriso_option_version(struct XorrisO *xorriso, int flag)
  sprintf(xorriso->result_line,
 "ISO 9660 Rock Ridge filesystem manipulator and CD/DVD/BD burn program\n");
  sprintf(xorriso->result_line+strlen(xorriso->result_line),
-"Copyright (C) 2011, Thomas Schmitt <scdbackup@gmx.net>, libburnia project.\n");
+"Copyright (C) 2013, Thomas Schmitt <scdbackup@gmx.net>, libburnia project.\n");
  Xorriso_result(xorriso, 0);
  sprintf(xorriso->result_line,
          "xorriso version   :  %d.%d.%d%s\n",
          Xorriso_header_version_majoR, Xorriso_header_version_minoR,
          Xorriso_header_version_micrO, Xorriso_program_patch_leveL);
- sprintf(xorriso->result_line+strlen(xorriso->result_line),
-         "Version timestamp :  %s\n",Xorriso_timestamP);
- sprintf(xorriso->result_line+strlen(xorriso->result_line),
+ Xorriso_result(xorriso, 0);
+ sprintf(xorriso->result_line, "Version timestamp :  %s\n",Xorriso_timestamP);
+ Xorriso_result(xorriso, 0);
+ sprintf(xorriso->result_line,
          "Build timestamp   :  %s\n",Xorriso_build_timestamP);
  Xorriso_result(xorriso, 0);
  Xorriso_report_lib_versions(xorriso, 0);
@@ -1837,6 +2028,26 @@ ex:;
 }
 
 
+/* Command -write_type */
+int Xorriso_option_write_type(struct XorrisO *xorriso, char *mode, int flag)
+{
+ if(strcmp(mode, "auto") == 0)
+   xorriso->do_tao = 0;
+ else if(strcmp(mode, "tao") == 0 || strcmp(mode, "TAO") == 0) 
+   xorriso->do_tao = 1;
+ else if(strcmp(mode, "sao") == 0 || strcmp(mode, "SAO") == 0 ||
+         strcmp(mode, "dao") == 0 || strcmp(mode, "DAO") == 0 ||
+         strcmp(mode, "sao/dao") == 0 || strcmp(mode, "SAO/DAO") == 0 ) 
+   xorriso->do_tao = -1;
+ else {
+   sprintf(xorriso->info_text, "-write_type: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -xattr "on"|"off" */
 int Xorriso_option_xattr(struct XorrisO *xorriso, char *mode, int flag)
 {
@@ -1888,7 +2099,9 @@ int Xorriso_option_zisofs(struct XorrisO *xorriso, char *mode, int flag)
      xorriso->zlib_level= num;
 
    } else if(strncmp(cpt, "ziso_used=", 10) == 0 ||
-             strncmp(cpt, "osiz_used=", 10) == 0) {
+             strncmp(cpt, "osiz_used=", 10) == 0 ||
+             strncmp(cpt, "gzip_used=", 10) == 0 ||
+             strncmp(cpt, "gunzip_used=", 12) == 0) {
      /* (ignored info from -status) */;
 
    } else if(strncmp(cpt, "block_size=", 11)==0) {
